@@ -55,6 +55,20 @@ fetch <- function(host, dataset) {
   message("Please use a function starts with fetch_")
 }
 
+check_hiplot <- function(host) {
+  use_hiplot <- getOption("use_hiplot", default = FALSE)
+  if (use_hiplot) {
+    if (!grepl("hiplot", host)) {
+      message("Use hiplot server (China) for mirrored data hubs (set 'options(use_hiplot = FALSE)' to disable it)")
+      host2 <- as.character(.xena_mirror_map_rv[host])
+      if (!is.na(host2)) {
+        host <- host2
+      }
+    }
+  }
+  return(host)
+}
+
 #' @describeIn fetch fetches values from a dense matrix.
 #' @export
 fetch_dense_values <- function(host, dataset, identifiers = NULL, samples = NULL,
@@ -65,6 +79,8 @@ fetch_dense_values <- function(host, dataset, identifiers = NULL, samples = NULL
     is.logical(check), is.logical(use_probeMap)
   )
   .attach_this()
+
+  host <- check_hiplot(host)
   if (check) {
     # obtain all samples
     all_samples <- fetch_dataset_samples(host, dataset)
@@ -207,6 +223,7 @@ fetch_sparse_values <- function(host, dataset, genes, samples = NULL,
   )
   .attach_this()
 
+  host <- check_hiplot(host)
   if (is.null(samples)) {
     samples <- fetch_dataset_samples(host, dataset)
   }
@@ -236,6 +253,7 @@ fetch_sparse_values <- function(host, dataset, genes, samples = NULL,
 #' @export
 fetch_dataset_samples <- function(host, dataset, limit = NULL) {
   .attach_this()
+  host <- check_hiplot(host)
   .p_dataset_samples(host = host, dataset = dataset, limit = limit)
 }
 
@@ -243,6 +261,7 @@ fetch_dataset_samples <- function(host, dataset, limit = NULL) {
 #' @export
 fetch_dataset_identifiers <- function(host, dataset) {
   .attach_this()
+  host <- check_hiplot(host)
   .p_dataset_field(host = host, dataset = dataset)
 }
 
